@@ -1,5 +1,6 @@
 """User entrypoint to elapid"""
 
+from elapid.__version__ import __doc__ as __version__
 from elapid.features import (
     CategoricalTransformer,
     HingeTransformer,
@@ -18,11 +19,11 @@ from elapid.geo import (
     sample_geoseries,
     sample_raster,
     sample_vector,
-    stack_geometries,
+    stack_geodataframes,
     xy_to_geoseries,
     zonal_stats,
 )
 from elapid.models import EnsembleModel, MaxentModel, NicheEnvelopeModel
 from elapid.stats import normalize_sample_probabilities
-from elapid.train_test_split import GeographicKFold, checkerboard_split
-from elapid.utils import load_object, load_sample_data, save_object
+from elapid.train_test_split import BufferedLeaveOneOut, GeographicKFold, checkerboard_split
+from elapid.utils import download_sample_data, load_object, load_sample_data, save_object
