@@ -83,9 +83,6 @@ class FeaturesMixin:
 class LinearTransformer(MinMaxScaler):
     """Applies linear feature transformations to rescale features from 0-1."""
 
-    clamp: bool = None
-    feature_range: None
-
     def __init__(
         self,
         clamp: bool = MaxentConfig.clamp,
@@ -99,10 +96,6 @@ class LinearTransformer(MinMaxScaler):
 class QuadraticTransformer(BaseEstimator, TransformerMixin):
     """Applies quadtratic feature transformations and rescales features from 0-1."""
 
-    clamp: bool = None
-    feature_range: Tuple[float, float] = None
-    estimator: BaseEstimator = None
-
     def __init__(
         self,
         clamp: bool = MaxentConfig.clamp,
@@ -110,6 +103,7 @@ class QuadraticTransformer(BaseEstimator, TransformerMixin):
     ):
         self.clamp = clamp
         self.feature_range = feature_range
+        self.estimator = None
 
     def fit(self, x: ArrayLike) -> "QuadraticTransformer":
         """Compute the minimum and maximum for scaling.
@@ -168,10 +162,6 @@ class QuadraticTransformer(BaseEstimator, TransformerMixin):
 class ProductTransformer(BaseEstimator, TransformerMixin):
     """Computes the column-wise product of an array of input features, rescaling from 0-1."""
 
-    clamp: bool = None
-    feature_range: Tuple[float, float] = None
-    estimator: BaseEstimator = None
-
     def __init__(
         self,
         clamp: bool = MaxentConfig.clamp,
@@ -179,6 +169,7 @@ class ProductTransformer(BaseEstimator, TransformerMixin):
     ):
         self.clamp = clamp
         self.feature_range = feature_range
+        self.estimator = None
 
     def fit(self, x: ArrayLike) -> "ProductTransformer":
         """Compute the minimum and maximum for scaling.
@@ -223,16 +214,13 @@ class ProductTransformer(BaseEstimator, TransformerMixin):
 
 
 class ThresholdTransformer(BaseEstimator, TransformerMixin):
-    """Applies binary thresholds to each covariate based on n evenly-spaced
-    thresholds across it's min/max range."""
-
-    n_thresholds_: int = None
-    mins_: np.ndarray = None
-    maxs_: np.ndarray = None
-    threshold_indices_: np.ndarray = None
+    """Apply binary thresholds across evenly-spaced bins for each covariate."""
 
     def __init__(self, n_thresholds: int = MaxentConfig.n_threshold_features):
-        self.n_thresholds_ = n_thresholds
+        self.n_thresholds = n_thresholds
+        self.mins_ = None
+        self.maxs_ = None
+        self.threshold_indices_ = None
 
     def fit(self, x: ArrayLike) -> "ThresholdTransformer":
         """Compute the minimum and maximum for scaling.
@@ -248,7 +236,7 @@ class ThresholdTransformer(BaseEstimator, TransformerMixin):
         x = np.array(x)
         self.mins_ = x.min(axis=0)
         self.maxs_ = x.max(axis=0)
-        self.threshold_indices_ = np.linspace(self.mins_, self.maxs_, self.n_thresholds_)
+        self.threshold_indices_ = np.linspace(self.mins_, self.maxs_, self.n_thresholds)
 
         return self
 
@@ -285,13 +273,11 @@ class ThresholdTransformer(BaseEstimator, TransformerMixin):
 class HingeTransformer(BaseEstimator, TransformerMixin):
     """Fits hinge transformations to an array of covariates."""
 
-    n_hinges_: int = None
-    mins_: np.ndarray = None
-    maxs_: np.ndarray = None
-    hinge_indices_: np.ndarray = None
-
     def __init__(self, n_hinges: int = MaxentConfig.n_hinge_features):
-        self.n_hinges_ = n_hinges
+        self.n_hinges = n_hinges
+        self.mins_ = None
+        self.maxs_ = None
+        self.hinge_indices_ = None
 
     def fit(self, x: ArrayLike) -> "HingeTransformer":
         """Compute the minimum and maximum for scaling.
@@ -307,7 +293,7 @@ class HingeTransformer(BaseEstimator, TransformerMixin):
         x = np.array(x)
         self.mins_ = x.min(axis=0)
         self.maxs_ = x.max(axis=0)
-        self.hinge_indices_ = np.linspace(self.mins_, self.maxs_, self.n_hinges_)
+        self.hinge_indices_ = np.linspace(self.mins_, self.maxs_, self.n_hinges)
 
         return self
 
@@ -322,7 +308,7 @@ class HingeTransformer(BaseEstimator, TransformerMixin):
             ndarray with transformed data.
         """
         x = np.array(x)
-        xarr = repeat_array(x, self.n_hinges_ - 1, axis=-1)
+        xarr = repeat_array(x, self.n_hinges - 1, axis=-1)
         lharr = repeat_array(self.hinge_indices_[:-1].transpose(), len(x), axis=0)
         rharr = repeat_array(self.hinge_indices_[1:].transpose(), len(x), axis=0)
         lh = left_hinge(xarr, lharr, self.maxs_)
@@ -346,10 +332,8 @@ class HingeTransformer(BaseEstimator, TransformerMixin):
 class CategoricalTransformer(BaseEstimator, TransformerMixin):
     """Applies one-hot encoding to categorical covariate datasets."""
 
-    estimators_: list = None
-
     def __init__(self):
-        pass
+        self.estimators_ = None
 
     def fit(self, x: ArrayLike) -> "CategoricalTransformer":
         """Compute the minimum and maximum for scaling.
@@ -423,25 +407,6 @@ class CumulativeTransformer(QuantileTransformer):
 class MaxentFeatureTransformer(BaseEstimator, TransformerMixin, FeaturesMixin):
     """Transforms covariate data into maxent-format feature data."""
 
-    feature_types: list = None
-    clamp: bool = None
-    n_hinge_features: int = None
-    n_threshold_features: int = None
-    categorical_: list = None
-    continuous_: list = None
-    categorical_pd_: list = None
-    continuous_pd_: list = None
-    labels_: list = None
-    estimators_: dict = {
-        "linear": None,
-        "quadratic": None,
-        "product": None,
-        "threshold": None,
-        "hinge": None,
-        "categorical": None,
-    }
-    feature_names_: list = None
-
     def __init__(
         self,
         feature_types: Union[str, list] = MaxentConfig.feature_types,
@@ -461,6 +426,20 @@ class MaxentFeatureTransformer(BaseEstimator, TransformerMixin, FeaturesMixin):
         self.clamp = clamp
         self.n_hinge_features = n_hinge_features
         self.n_threshold_features = n_threshold_features
+        self.categorical_ = None
+        self.continuous_ = None
+        self.categorical_pd_ = None
+        self.continuous_pd_ = None
+        self.labels_ = None
+        self.feature_names_ = None
+        self.estimators_ = {
+            "linear": None,
+            "quadratic": None,
+            "product": None,
+            "threshold": None,
+            "hinge": None,
+            "categorical": None,
+        }
 
     def fit(self, x: ArrayLike, categorical: list = None, labels: list = None) -> "MaxentFeatureTransformer":
         """Compute the minimum and maximum for scaling.
@@ -507,13 +486,13 @@ class MaxentFeatureTransformer(BaseEstimator, TransformerMixin, FeaturesMixin):
             estimator = ThresholdTransformer(n_thresholds=self.n_threshold_features)
             estimator.fit(con)
             self.estimators_["threshold"] = estimator
-            feature_names += ["threshold"] * (estimator.n_thresholds_ * ncols)
+            feature_names += ["threshold"] * (estimator.n_thresholds * ncols)
 
         if "hinge" in self.feature_types:
             estimator = HingeTransformer(n_hinges=self.n_hinge_features)
             estimator.fit(con)
             self.estimators_["hinge"] = estimator
-            feature_names += ["hinge"] * ((estimator.n_hinges_ - 1) * 2 * ncols)
+            feature_names += ["hinge"] * ((estimator.n_hinges - 1) * 2 * ncols)
 
         if cat is not None:
             estimator = CategoricalTransformer()
