@@ -24,16 +24,16 @@ from elapid.utils import (
     get_raster_band_indexes,
     get_tqdm,
     n_digits,
+    tqdm_opts,
 )
 
 tqdm = get_tqdm()
-tqdm_opts = {"bar_format": "{l_bar}{bar:30}{r_bar}{bar:-30b}"}
 
 # sampling tools
 
 
 def xy_to_geoseries(
-    x: Union[float, list, np.ndarray], y: Union[float, list, np.ndarray], crs: CRSType = "epsg:4236"
+    x: Union[float, list, np.ndarray], y: Union[float, list, np.ndarray], crs: CRSType = "epsg:4326"
 ) -> gpd.GeoSeries:
     """Converts x/y data into a geopandas geoseries.
 
@@ -56,7 +56,7 @@ def xy_to_geoseries(
     return gs
 
 
-def stack_geometries(
+def stack_geodataframes(
     presence: Vector, background: Vector, add_class_label: bool = False, target_crs: str = "presence"
 ) -> gpd.GeoDataFrame:
     """Concatenate geometries from two GeoSeries/GeoDataFrames.
@@ -71,29 +71,39 @@ def stack_geometries(
     Returns:
         merged GeoDataFrame with all geometries projected to the same crs.
     """
-    pgeo = presence.geometry
-    bgeo = background.geometry
+    validate_gpd(presence)
+    validate_gpd(background)
 
+    # cast to geodataframes
+    if isinstance(presence, gpd.GeoSeries):
+        presence = presence.to_frame("geometry")
+    if isinstance(background, gpd.GeoSeries):
+        background = background.to_frame("geometry")
+
+    # handle projection mismatch
     crs = presence.crs
-    if not crs_match(presence.crs, background.crs):
+    if crs_match(presence.crs, background.crs):
+        # explicitly set the two to exactly matching crs as geopandas
+        # throws errors if there's any mismatch at all
+        background.crs = presence.crs
+    else:
         if target_crs.lower() == "presence":
-            bgeo = bgeo.to_crs(crs)
+            background.to_crs(crs, inplace=True)
         elif target_crs.lower() == "background":
             crs = background.crs
-            pgeo = pgeo.to_crs(crs)
+            presence.to_crs(crs, inplace=True)
         else:
             raise NameError(f"Unrecognized target_crs option: {target_crs}")
 
-    attributes = {}
     if add_class_label:
-        npresence = len(pgeo)
-        nbackground = len(bgeo)
-        classes = np.zeros(npresence + nbackground, dtype="uint8")
-        classes[:npresence] = 1
-        attributes["class"] = classes
+        presence["class"] = 1
+        background["class"] = 0
 
-    geometry = pd.concat((pgeo, bgeo), axis=0, ignore_index=True)
-    gdf = gpd.GeoDataFrame(attributes, geometry=geometry, crs=crs)
+    matching = [col for col in presence.columns if col in background.columns]
+    assert len(matching) > 0, "no matching columns found between data frames"
+
+    merged = pd.concat((presence[matching], background[matching]), axis=0, ignore_index=True)
+    gdf = gpd.GeoDataFrame(merged, crs=crs)
 
     return gdf
 
@@ -114,7 +124,6 @@ def sample_raster(raster_path: str, count: int, nodata: float = None, ignore_mas
     """
     # handle masked vs unmasked data differently
     with rio.open(raster_path) as src:
-
         if src.nodata is None or ignore_mask:
             if nodata is None:
                 xmin, ymin, xmax, ymax = src.bounds
@@ -159,7 +168,6 @@ def sample_bias_file(raster_path: str, count: int, ignore_mask: bool = False) ->
         points: Point geometry geoseries
     """
     with rio.open(raster_path) as src:
-
         if src.nodata is None or ignore_mask:
             data = src.read(1)
             rows, cols = np.where(data)
@@ -305,6 +313,7 @@ def annotate(
     raster_paths: Union[str, list],
     labels: list = None,
     drop_na: bool = True,
+    quiet: bool = False,
 ) -> gpd.GeoDataFrame:
     """Read raster values for each point in a vector and append as new columns.
 
@@ -316,6 +325,7 @@ def annotate(
         labels: band name labels. number of labels should match the
             total number of bands across all raster_paths.
         drop_na: drop all records with no-data values.
+        quiet: silence progress bar output.
 
     Returns:
         GeoDataFrame annotated with the pixel values from each raster
@@ -326,30 +336,41 @@ def annotate(
 
     # read raster values based on the points dtype
     if isinstance(points, gpd.GeoSeries):
+        points = points.reset_index(drop=True)
         gdf = annotate_geoseries(
             points,
             raster_paths,
             labels=labels,
             drop_na=drop_na,
+            quiet=quiet,
         )
 
     elif isinstance(points, gpd.GeoDataFrame) or isinstance(points, pd.DataFrame):
+        points = points.reset_index(drop=True)
         gdf = annotate_geoseries(
             points.geometry,
             raster_paths,
             labels=labels,
             drop_na=drop_na,
+            quiet=quiet,
         )
 
         # append annotations to the input dataframe
         gdf = pd.concat([points, gdf.drop(["geometry"], axis=1, errors="ignore")], axis=1)
 
     elif os.path.isfile(points):
-        gdf = annotate_vector(points, raster_paths, labels=labels, drop_na=drop_na)
+        gdf = annotate_vector(points, raster_paths, labels=labels, drop_na=drop_na, quiet=quiet)
 
     else:
         raise TypeError("points arg must be a valid path, GeoDataFrame, or GeoSeries")
 
+    if drop_na:
+        try:
+            valid = gdf["valid"] == 1
+            gdf = gdf[valid].drop(columns="valid").dropna().reset_index(drop=True)
+        except KeyError:
+            pass
+
     return gdf
 
 
@@ -358,6 +379,7 @@ def annotate_vector(
     raster_paths: list,
     labels: list = None,
     drop_na: bool = True,
+    quiet: bool = False,
 ) -> gpd.GeoDataFrame:
     """Reads and stores pixel values from rasters using a point-format vector file.
 
@@ -366,6 +388,7 @@ def annotate_vector(
         raster_paths: raster paths to extract pixel values from
         labels: band name labels. should match the total number of bands across all raster_paths
         drop_na: drop all records with no-data values
+        quiet: silence progress bar output.
 
     Returns:
         gdf: GeoDataFrame annotated with the pixel values from each raster
@@ -377,12 +400,18 @@ def annotate_vector(
     gdf = gpd.read_file(vector_path)
     raster_df = annotate_geoseries(gdf.geometry, raster_paths, labels, drop_na)
     gdf = pd.concat([gdf, raster_df.drop(["geometry"], axis=1, errors="ignore")], axis=1)
+
     return gdf
 
 
 def annotate_geoseries(
-    points: gpd.GeoSeries, raster_paths: list, labels: list = None, drop_na: bool = True, dtype: str = None
-) -> gpd.GeoDataFrame:
+    points: gpd.GeoSeries,
+    raster_paths: list,
+    labels: list = None,
+    drop_na: bool = True,
+    dtype: str = None,
+    quiet: bool = False,
+) -> (gpd.GeoDataFrame, np.ndarray):
     """Reads and stores pixel values from rasters using point locations.
 
     Args:
@@ -391,6 +420,7 @@ def annotate_geoseries(
         labels: band labels. must match the total number of bands for all raster_paths.
         drop_na: drop records with no-data values.
         dtype: output column data type. uses the first raster's dtype by default.
+        quiet: silence progress bar output.
 
     Returns:
         gdf: GeoDataFrame annotated with the pixel values from each raster
@@ -408,9 +438,10 @@ def annotate_geoseries(
     nodata_flag = False
 
     # annotate each point with the pixel values for each raster
-    for raster_idx, raster_path in tqdm(enumerate(raster_paths), desc="Raster", total=n_rasters, **tqdm_opts):
+    for raster_idx, raster_path in tqdm(
+        enumerate(raster_paths), desc="Raster", total=n_rasters, disable=quiet, **tqdm_opts
+    ):
         with rio.open(raster_path, "r") as src:
-
             # reproject points to match raster and convert to a dataframe
             if not crs_match(points.crs, src.crs):
                 points = points.to_crs(src.crs)
@@ -425,25 +456,33 @@ def annotate_geoseries(
             # read each pixel value
             n_points = len(points)
             samples_iter = list(
-                tqdm(src.sample(xys, masked=False), desc="Sample", total=n_points, leave=False, **tqdm_opts)
+                tqdm(
+                    src.sample(xys, masked=False),
+                    desc="Sample",
+                    total=n_points,
+                    leave=False,
+                    disable=quiet,
+                    **tqdm_opts,
+                )
             )
             samples = np.array(samples_iter, dtype=dtype)
+            raster_values.append(samples)
 
             # identify nodata points to remove later
             if drop_na and src.nodata is not None:
                 nodata_flag = True
                 valid_idxs.append(samples[:, 0] != src.nodata)
 
-            raster_values.append(samples)
-
     # merge the arrays from each raster
     values = np.concatenate(raster_values, axis=1, dtype=dtype)
 
     if nodata_flag:
-        valid = np.all(valid_idxs, axis=0)
-        values = values[valid, :]
-        points = points.iloc[valid]
-        points.index = range(valid.sum())
+        valid = np.all(valid_idxs, axis=0).reshape(-1, 1)
+        values = np.concatenate([values, valid], axis=1, dtype=dtype)
+        labels.append("valid")
+        # values = values[valid, :]
+        # points = points.iloc[valid]
+        # points.index = range(valid.sum())
 
     # convert to a geodataframe
     gdf = gpd.GeoDataFrame(values, geometry=points.geometry, columns=labels)
@@ -509,6 +548,7 @@ def apply_model_to_rasters(
     windowed: bool = True,
     predict_proba: bool = False,
     ignore_sklearn: bool = True,
+    quiet: bool = False,
     **kwargs,
 ) -> None:
     """Applies a trained model to a list of raster datasets.
@@ -539,6 +579,7 @@ def apply_model_to_rasters(
             slower, but more memory efficient
         predict_proba: use model.predict_proba() instead of model.predict()
         ignore_sklearn: silence sklearn warning messages
+        quiet: silence progress bar output
         **kwargs: additonal keywords to pass to model.predict()
 
     Returns:
@@ -589,8 +630,7 @@ def apply_model_to_rasters(
 
     # read and reproject blocks from each data source and write predictions to disk
     with rio.open(output_path, "w", **dst_profile) as dst:
-        for window in tqdm(windows, desc="Window", **tqdm_opts):
-
+        for window in tqdm(windows, desc="Window", disable=quiet, **tqdm_opts):
             # create stacked arrays to handle multi-raster, multi-band inputs
             # that may have different nodata locations
             covariates = np.zeros((nbands, window.height, window.width), dtype=np.float32)
@@ -704,6 +744,7 @@ def zonal_stats(
     mode: bool = False,
     all: bool = False,
     percentiles: list = [],
+    quiet: bool = False,
 ) -> gpd.GeoDataFrame:
     """Compute raster summary stats for each polygon in a GeoSeries or GeoDataFrame.
 
@@ -717,6 +758,7 @@ def zonal_stats(
             set to True to compute these stats
         all: compute all of the above stats
         percentiles: list of 0-100 percentile ranges to compute
+        quiet: silence progress bar output
 
     Returns:
         GeoDataFrame with zonal stats for each raster band in new columns.
@@ -756,8 +798,7 @@ def zonal_stats(
     raster_dfs = []
 
     # run zonal stats raster-by-raster (instead of iterating first over geometries)
-    for r, raster in tqdm(enumerate(raster_paths), total=len(raster_paths), desc="Raster", **tqdm_opts):
-
+    for r, raster in tqdm(enumerate(raster_paths), total=len(raster_paths), desc="Raster", disable=quiet, **tqdm_opts):
         # format the band labels
         band_labels = labels[band_idx[r] : band_idx[r + 1]]
         n_raster_bands = band_idx[r + 1] - band_idx[r]
@@ -767,7 +808,6 @@ def zonal_stats(
 
         # open the raster for reading
         with rio.open(raster, "r") as src:
-
             # reproject the polygon data as necessary
             if not crs_match(polys.crs, src.crs):
                 polys = polys.to_crs(src.crs)
@@ -779,7 +819,9 @@ def zonal_stats(
                 stats_arrays.append(np.zeros((len(polys), n_raster_bands), dtype=dtype))
 
             # iterate over each geometry to read data and compute stats
-            for p, poly in tqdm(enumerate(polys), total=len(polys), desc="Polygon", leave=False, **tqdm_opts):
+            for p, poly in tqdm(
+                enumerate(polys), total=len(polys), desc="Polygon", leave=False, disable=quiet, **tqdm_opts
+            ):
                 data = read_raster_from_polygon(src, poly)
                 for method, array in zip(stats_methods, stats_arrays):
                     array[p, :] = method.reduce(data)
@@ -810,7 +852,8 @@ def nearest_point_distance(
         points2: return the closest distance *to* these points
             if None, compute the distance to the nearest points
             in the points1 series
-        n_neighbors: compute the average distance to the nearest n_neighbors
+        n_neighbors: compute the average distance to the nearest n_neighbors.
+            set to -1 to compute the distance to all neighbors.
         cpu_count: number of cpus to use for estimation.
             -1 uses all cores
 
@@ -833,6 +876,9 @@ def nearest_point_distance(
         if not crs_match(points1.crs, points2.crs):
             warnings.warn("CRS mismatch between points")
 
+    if n_neighbors < 1:
+        n_neighbors = len(pta2) - k_offset
+
     tree = KDTree(pta1)
     k = np.arange(n_neighbors) + k_offset
     distance, idx = tree.query(pta2, k=k, workers=cpu_count)
@@ -840,14 +886,15 @@ def nearest_point_distance(
     return distance.mean(axis=1)
 
 
-def distance_weights(points: Vector, n_neighbors: int = 1, center: str = "median", cpu_count: int = -1) -> np.ndarray:
+def distance_weights(points: Vector, n_neighbors: int = -1, center: str = "median", cpu_count: int = -1) -> np.ndarray:
     """Compute sample weights based on the distance between points.
 
     Assigns higher scores to isolated points, lower scores to clustered points.
 
     Args:
         points: point-format GeoSeries or GeoDataFrame
-        n_neighbors: compute the average distance to the nearest n_neighbors
+        n_neighbors: compute weights based on average distance to the nearest n_neighbors
+            set to -1 to compute the distance to all neighbors.
         center: rescale the weights to center the mean or median of the array on 1
             accepts either 'mean' or 'median' as input.
             pass None to ignore.
